require 'service_pack/service_collection'

module ServicePack
  module Core
    def self.included(cls)
      cls.send :extend, ClassMethods
      cls.send :include, InstanceMethods
    end

    module ClassMethods
      def register(service_name, service_class)
        services.add service_name, service_class

        define_service_accessor service_name
      end

      def registered?(service_name)
        services.added? service_name
      end

      private
      def define_service_accessor(service_name)
        define_method service_name do
          call_service(service_name)
        end
      end

      def services
        @services ||= ServiceCollection.new
      end
    end

    module InstanceMethods
      def [](service_name)
        call_service service_name
      end

      private
      def services
        self.class.send :services
      end

      def call_service(service_name)
        cache service_name do
          services.instanciate(self, service_name)
        end
      end

      def cache(key, &block)
        @cache ||= {}
        @cache[key] ||= block.call
      end
    end
  end
end
